package models

import (
	"fmt"
	"github.com/jinzhu/gorm"
	_ "github.com/jinzhu/gorm/dialects/mysql"
	"log"
	"time"

	"github.com/iamlockon/gorestemplate/pkg/setting"
)

var db *gorm.DB

type Model struct {
	ID         int `gorm:"primary_key" json:"id"`
	CreatedOn  int `json:"created_on"`
	ModifiedOn int `json:"modified_on"`
	// DeletedOn int `json:"deleted_on"` // Uncomment this line if you need soft delete.
}

func Setup() {
	var err error
	db, err = gorm.Open(setting.DatabaseSetting.Type, fmt.Sprintf("%s:%s@tcp(%s)/%s?charset=utf8&parseTime=True&loc=Local",
		setting.DatabaseSetting.User,
		setting.DatabaseSetting.Password,
		setting.DatabaseSetting.Host,
		setting.DatabaseSetting.Name))

	if err != nil {
		log.Fatalf("models.Setup err: %v", err)
	}

	gorm.DefaultTableNameHandler = func(db *gorm.DB, defaultTableName string) string {
		return setting.DatabaseSetting.TablePrefix + defaultTableName
	}

	db.SingularTable(true)
	db.Callback().Create().Replace("gorm:update_time_stamp", updateTimeStampForCreateCallback)
	db.Callback().Update().Replace("gorm:update_time_stamp", updateTimeStampForUpdateCallback)
	// db.Callback().Delete().Replace("gorm:delete", deleteCallback)
	db.DB().SetMaxIdleConns(10)
	db.DB().SetMaxOpenConns(100)
}

// updateTimeStampForCreateCallback will set `CreatedOn`, `ModifiedOn` when creating
func updateTimeStampForCreateCallback(scope *gorm.Scope) {
	if !scope.HasError() {
		nowTime := time.Now().Unix()
		if createTimeField, ok := scope.FieldByName("CreatedOn"); ok {
			if createTimeField.IsBlank {
				createTimeField.Set(nowTime)
			}
		}

		if modifyTimeField, ok := scope.FieldByName("ModifiedOn"); ok {
			if modifyTimeField.IsBlank {
				modifyTimeField.Set(nowTime)
			}
		}
	}
}

// updateTimeStampForUpdateCallback will set `ModifyTime` when updating
func updateTimeStampForUpdateCallback(scope *gorm.Scope) {
	if _, ok := scope.Get("gorm:update_column"); !ok {
		scope.SetColumn("ModifiedOn", time.Now().Unix())
	}
}

// Uncomment following lines if you need soft delete.
// func deleteCallback(scope *gorm.Scope) {
//     if !scope.HasError() {
//         var extraOption string
//         if str, ok := scope.Get("gorm:delete_option"); ok {
//             extraOption = fmt.Sprint(str)
//         }

//         deletedOnField, hasDeletedOnField := scope.FieldByName("DeletedOn")

//         if !scope.Search.Unscoped && hasDeletedOnField {
//             scope.Raw(fmt.Sprintf(
//                 "UPDATE %v SET %v=%v%v%v",
//                 scope.QuotedTableName(),
//                 scope.Quote(deletedOnField.DBName),
//                 scope.AddToVars(time.Now().Unix()),
//                 addExtraSpaceIfExist(scope.CombinedConditionSql()),
//                 addExtraSpaceIfExist(extraOption),
//             )).Exec()
//         } else {
//             scope.Raw(fmt.Sprintf(
//                 "DELETE FROM %v%v%v",
//                 scope.QuotedTableName(),
//                 addExtraSpaceIfExist(scope.CombinedConditionSql()),
//                 addExtraSpaceIfExist(extraOption),
//             )).Exec()
//         }
//     }
// }
// func addExtraSpaceIfExist(str string) string {
//     if str != "" {
//         return " " + str
//     }
//     return ""
// }

func CloseDB() {
	defer db.Close()
}
